@@ -25,6 +25,8 @@ SQLITE3_INDEX_FILE="index.db"
 FRAME_NUMBER_RESET=True
 MULTI_STORE_ENABLED=False
 COLOR=True
-SELECTED_STORE="lowres/metadata.yaml"
+SELECTED_STORE="highspeed"
 LOOKUP_NEAREST=False
-LOGGING_FILE=os.path.join(os.environ["HOME"], ".config", "imgstore", "logging.yml")
\ No newline at end of file
+LOGGING_FILE=os.path.join(os.environ["HOME"], ".config", "imgstore", "logging.yml")
+FIRST_FEED="master"
+RESHAPE_METHOD="resize"
\ No newline at end of file
