@@ -5,6 +5,7 @@ import glob
 import operator
 import warnings
 
+import codetiming
 import cv2
 import numpy as np
 from imgstore.constants import STORE_MD_KEY, VERBOSE_DEBUG_CHUNKS
@@ -13,14 +14,16 @@ from imgstore.stores.utils.formats import get_formats
 from imgstore.stores.base import _ImgStore
 
 try:
-    import cv2cuda
+    import cv2cuda # type: ignore
     CV2CUDA_AVAILABLE=True
 except Exception:
     cv2cuda=None
     CV2CUDA_AVAILABLE=False
 
 isColor=False
+ONLY_ALLOW_EVEN_SIDES=True
 
+logger = logging.getLogger(__name__)
 
 def find_chunks_video(basedir, ext, chunk_numbers=None):
     if chunk_numbers is None:
@@ -38,7 +41,9 @@ class VideoImgStore(_ImgStore):
     def __init__(self, **kwargs):
 
         self._cap = None
+        self._cap_hq = None
         self._capfn = None
+        self._capfn_hq = None
         self._last_capfn=None
 
 
@@ -137,11 +142,12 @@ class VideoImgStore(_ImgStore):
 
     def _save_image(self, img, frame_number, frame_time):
         # we always write color because its more supported
+            
         if self._color:
             frame = ensure_color(img)
         else:
             frame = ensure_grayscale(img)
-    
+
         self._cap.write(frame)
         if self._chunk_current_frame_idx > 0 and not os.path.isfile(self._capfn):
             raise Exception(
@@ -156,6 +162,7 @@ class VideoImgStore(_ImgStore):
     def _save_chunk(self, old, new):
         if self._cap is not None:
             self._cap.release()
+            if self._cap_hq is not None: self._cap_hq.release()
             self._save_chunk_metadata(os.path.join(self._basedir, '%06d' % old))
 
         if new is not None:
@@ -167,22 +174,29 @@ class VideoImgStore(_ImgStore):
                     self._codec=self._cv2_fmts['avc1/mp4']
 
 
-                if self._codec == "h264_nvenc":
+                if self._codec == "h264_nvenc" and new != 0:
                     self._cap = cv2cuda.VideoWriter(
                         filename=fn,
                         apiPreference="FFMPEG",
                         fourcc="h264_nvenc",
-                        fps=self._fps,
+                        fps=self.fps,
                         frameSize=(w, h),
                         isColor=self._color
                     )
             
                 else:
+                    if new == 0 and self._codec == "h264_nvenc":
+                        codec = cv2.VideoWriter_fourcc(*"DIVX")
+                        filename = fn.replace(".mp4", ".avi")
+                    else:
+                        codec = self._codec
+                        filename = fn
+
                     self._cap = cv2.VideoWriter(
-                        filename=fn,
+                        filename=filename,
                         apiPreference=cv2.CAP_FFMPEG,
-                        fourcc=self._codec,
-                        fps=self._fps,
+                        fourcc=codec,
+                        fps=self.fps,
                         frameSize=(w, h),
                         isColor=self._color
                     )
@@ -201,55 +215,95 @@ class VideoImgStore(_ImgStore):
                 self._cap = cv2.VideoWriter(
                     filename=fn,
                     fourcc=self._codec,
-                    fps=self._fps,
+                    fps=self.fps,
                     frameSize=(w, h),
                     isColor=self._color
                 )
 
             self._capfn = fn
             self._new_chunk_metadata(os.path.join(self._basedir, '%06d' % new))
+            self.frame_idx = 0
 
 
     @property
     def burn_in_period(self):
         pass
-        if self._capfn.endswith(".mp4") and os.path.exists(self._capfn_hq):
-            return self._fps * 5
+        if self._capfn.endswith(".mp4") and os.path.exists(self._capfn_hq) and self._cap_hq is not None:
+            return self.fps * 5
         else:
             return -1
 
+    @staticmethod
+    def _read(cap):
+        ret, img = cap.read()
+        if not ret:
+            return ret, None
+        if ONLY_ALLOW_EVEN_SIDES:
+            dims = list(img.shape)
+            for i, dimension in enumerate(img.shape):
+                if dimension % 2 != 0:
+                    dims[i] -= 1
+            
+            img = img[:dims[0], :dims[1]].copy(order="C")
+        return ret, img               
+
+
 
     def _load_image(self, idx):
 
-        if idx < self.burn_in_period:
-            cap  = self._cap_hq
-        else:
-            cap = self._cap
-        if self._supports_seeking:
-            # only seek if we have to, otherwise take the fast path
-            if (idx - self._chunk_current_frame_idx) != 1:
-                cap.set(getattr(cv2, "CAP_PROP_POS_FRAMES", 1), idx)
-        else:
-            raise Exception("Only videos with seeking support enabled")
-            # TODO
-            # Return back support for videos without seeking
-            # support by dealing here both with _cap and _cap_hq 
-            # if idx <= self._chunk_current_frame_idx:
-            #     self._load_chunk(self._chunk_n, _force=True)
 
-            # i = self._chunk_current_frame_idx + 1
-            # while i < idx:
-            #     _, img = cap.read()
-            #     i += 1
+        with codetiming.Timer(text="Reading image took {milliseconds:.0f} ms", logger=logger.debug):
 
-        ret, _img = cap.read()
-        assert ret, f"Cannot read frame from {self._capfn}"
-        if self._color:
-            # almost certainly no-op as opencv usually returns color frames....
-            img = ensure_color(_img)
-        else:
-            img = ensure_grayscale(_img)
-        self._last_img = img.copy()
+            if idx < self.burn_in_period:
+                cap  = self._cap_hq
+            else:
+                cap = self._cap
+            
+            if (idx - self._chunk_current_frame_idx) == 0 and self._last_img is not None:
+                ret = True
+                _img = self._last_img.copy()
+
+            else:
+                if self._supports_seeking:
+                    # only seek if we have to, otherwise take the fast path
+                    if (idx - self._chunk_current_frame_idx) != 1:
+                        # print(f"{idx - self._chunk_current_frame_idx} frames in the future")
+                        cap.set(getattr(cv2, "CAP_PROP_POS_FRAMES", 1), idx)
+                else:
+                    raise Exception("Only videos with seeking support enabled")
+                    # TODO
+                    # Return back support for videos without seeking
+                    # support by dealing here both with _cap and _cap_hq 
+                    # if idx <= self._chunk_current_frame_idx:
+                    #     self._load_chunk(self._chunk_n, _force=True)
+
+                    # i = self._chunk_current_frame_idx + 1
+                    # while i < idx:
+                    #     _, img = cap.read()
+                    #     i += 1
+
+                ret, _img = self._read(cap)
+                if not ret:
+                    return None, (None, None)
+                if cap is self._cap_hq:
+                    self._cap.set(getattr(cv2, "CAP_PROP_POS_FRAMES", 1), idx+1)
+
+            assert ret, f"Cannot read frame from {self._capfn}"
+
+        with codetiming.Timer(text="Ensuring color or grayscale took {milliseconds:.0f} ms", logger=logger.debug):
+            if self._color:
+                # almost certainly no-op as opencv usually returns color frames....
+                img = ensure_color(_img)
+            else:
+                img = ensure_grayscale(_img)
+
+        if self._metadata.get("apply_blur", False):
+            with codetiming.Timer(text="Applying gaussian blur took {milliseconds:.0f} ms", logger=logger.debug):
+                # print("Applying gaussian blur")
+                img = cv2.GaussianBlur(img, (0, 0), self._metadata["apply_blur"])
+
+        with codetiming.Timer(text="Copying image took {milliseconds:.0f} ms", logger=logger.debug):
+            self._last_img = img.copy()
 
         return img, (self._chunk_md['frame_number'][idx], self._chunk_md['frame_time'][idx])
 
@@ -258,22 +312,34 @@ class VideoImgStore(_ImgStore):
         if _force or (fn != self._capfn):
             if self._cap is not None:
                 self._cap.release()
+            if self._cap_hq is not None:
+                self._cap_hq.release()
 
             self._log.debug('loading chunk %s' % n)
             self._capfn = fn
-            self._cap = cv2.VideoCapture(self._capfn)
-            caps = [self._cap]
+
+            caps = []
+            fns = []
+
+            if os.path.exists(self._capfn):
+                self._cap = cv2.VideoCapture(self._capfn)
+                caps.append(self._cap)
+                fns.append(self._capfn)
 
             self._capfn_hq = os.path.splitext(fn)[0] + ".avi"
-            if self.burn_in_period > 0:
-                self._cap_hq = cv2.VideoCapture(self._capfn_hq)
-                caps.append(self._cap_hq)
+            self._cap_hq = cv2.VideoCapture(self._capfn_hq)
+            caps.append(self._cap_hq)
+            fns.append(self._capfn_hq)
 
             self._chunk_current_frame_idx = -1
 
-            for cap in caps:
+            for fn, cap in zip(fns, caps):
                 if not cap.isOpened():
-                    raise Exception("OpenCV unable to open %s" % fn)
+                    if fn == self._capfn_hq:
+                        self._log.warning(f"Cannot open {fn}")
+                        self._cap_hq = None
+                    else:
+                        raise Exception("OpenCV unable to open %s" % fn)
 
             self._chunk_md = self._index.get_chunk_metadata(n)
 
@@ -289,8 +355,7 @@ class VideoImgStore(_ImgStore):
     def supports_format(cls, fmt):
         return fmt in cls._cv2_fmts
 
-    @staticmethod
-    def _extract_only_frame(basedir, chunk_n, frame_n, smd):
+    def _extract_only_frame(self, basedir, chunk_n, frame_n, smd):
         capfn = os.path.join(basedir, '%06d%s' % (chunk_n,
                                                   VideoImgStore._get_chunk_extension(smd)))
         # noinspection PyArgumentList
@@ -303,7 +368,7 @@ class VideoImgStore(_ImgStore):
         try:
             if frame_n > 0:
                 cap.set(getattr(cv2, "CAP_PROP_POS_FRAMES", 1), frame_n)
-            _, img = cap.read()
+            _, img = self._read(cap)
             return img
         finally:
             cap.release()
@@ -319,6 +384,9 @@ class VideoImgStore(_ImgStore):
             self._cap = None
             self._last_capfn=self._capfn
             self._capfn = None
+        if self._cap_hq is not None:
+            self._cap_hq.release()
+            self._cap_fn = None
 
     def empty(self):
         _ImgStore.empty(self)
